from qtpy import QtGui, QtCore, QtWidgets
import numpy as np
from .galvoGraphics import *
import sys
from . import global_vars as g

def byref_str(s):
	return "ref(%s)" % s

if not g.DEBUG:
	try:
		from PyDAQmx import *
		from PyDAQmx.DAQmxCallBack import *
	except:
		print("Could not import PyDAQmx. Running in Debug mode")
		g.DEBUG = True

if g.DEBUG:
	DAQmx_Val_Volts = 'DAQmx_Val_Volts'
	DAQmx_Val_ChanForAllLines = 'DAQmx_Val_ChanForAllLines'
	DAQmx_Val_GroupByChannel = "DAQmx_Val_GroupByChannel"
	byref = byref_str

import time

class Laser(QtCore.QObject):
	sigRenamed = QtCore.Signal(str)
	def __init__(self, name, pin):
		super(Laser, self).__init__()
		self.name = name
		self.pin = pin
		self.active = False
	def setActive(self, a):
		self.active = a
	def rename(self, s):
		self.name = s
		self.sigRenamed.emit(s)
	def changePin(self, p):
		self.pin = p
	
class GalvoScene(QtWidgets.QGraphicsScene):
	def __init__(self, **kargs):
		super(GalvoScene, self).__init__(**kargs)
		self.galvo = GalvoDriver()
		self.crosshair = CrossHair()
		self.crosshair.sigMoved.connect(self.crosshairMoved)
		self.addItem(self.crosshair)
		self.line = GalvoStraightLine(QtCore.QPointF(-100, 0), QtCore.QPointF(0, 0))
		self.line.setVisible(False)
		self.addItem(self.line)
		self.drawMethod = 'Point'
		self.rois = []
		self.currentROI = None

	def crosshairMoved(self, pos):
		pos = self.mapToGalvo(pos)
		if not (0 <= pos.x() <= 1 and 0 <= pos.y() <= 1):
			pos.setX(min(1, max(0, pos.x())))
			pos.setY(min(1, max(0, pos.y())))
			self.crosshair.setPos(self.views()[0].mapToScene(self.views()[0].width() * pos.x(), self.views()[0].height() * pos.y()))
		self.galvo.moveTo(pos)

	def setDrawMethod(self, method):
		self.drawMethod = method
		self.crosshair.setVisible(method == 'Point')
		for roi in self.rois:
			roi.setVisible(method == 'ROI')
		self.line.setVisible(method == 'Line' and self.line.start.x() != -100)

		g.ui.laser1Button.setEnabled(method == 'Point')
		g.ui.laser2Button.setEnabled(method == 'Point')


		g.ui.intervalTypeLabel.setText("Line Intervals:" if method == 'Line' else 'Mesh Size:')

	def mousePressEvent(self, ev):
		if self.drawMethod =='Point':
			self.crosshair.setPos(ev.scenePos())
			self.crosshair.dragging = ev.button() == QtCore.Qt.RightButton
		else:
			if ev.button() == QtCore.Qt.RightButton:  # if right button, enable crosshair movement
				if self.drawMethod =='ROI':
					for i in range(len(self.rois) - 1, -1, -1):
						if self.rois[i].mouseOver(ev.scenePos()):
							self.removeItem(self.rois[i])
							self.rois.pop(i)
			elif ev.button() == QtCore.Qt.LeftButton:
				if self.drawMethod == "ROI":
					self.currentROI = GalvoShape(ev.scenePos())
					self.addItem(self.currentROI)
				elif self.drawMethod == "Line":
					self.line.setVisible(True)
					if not self.line.midDraw:
						self.line.setStart(ev.scenePos())
					else:
						self.line.setEnd(ev.scenePos(), finish=True)
		QtWidgets.QGraphicsScene.mousePressEvent(self, ev)

	def mouseMoveEvent(self, ev):
		if self.drawMethod == 'Point' and self.crosshair.dragging:
			self.crosshair.setPos(ev.scenePos())
		elif isinstance(self.currentROI, GalvoShape):
			self.currentROI.addPoint(ev.scenePos())
		elif self.line.midDraw:
			self.line.setEnd(ev.scenePos())

		QtWidgets.QGraphicsScene.mouseMoveEvent(self, ev)

	def mouseReleaseEvent(self, ev):
		self.crosshair.dragging = False
		if ev.button() == QtCore.Qt.RightButton:
			pass
		elif ev.button() == QtCore.Qt.LeftButton:
			if isinstance(self.currentROI, GalvoShape):
				self.currentROI.close()
				if self.currentROI.path.length() > 50:
					self.rois.append(self.currentROI)
				else:
					self.removeItem(self.currentROI)
			if self.line.midDraw:
				if (ev.scenePos() - self.line.start).manhattanLength() > 5:
					self.line.setEnd(ev.scenePos(), finish=True)

		self.currentROI = None
		QtWidgets.QGraphicsScene.mouseReleaseEvent(self, ev)

	def center(self):
		self.setDrawMethod("Point")
		self.crosshair.setPos(self.views()[0].mapToScene(self.views()[0].width() * .5, self.views()[0].height() * .5))

	def clear(self):
		for i in self.rois:
			if isinstance(i, GalvoLine):
				self.removeItem(i)
		self.rois = []

	def resetBounds(self):
		self.galvo.setBounds(QtCore.QRect(-10, -10, 20, 20))
		self.center()

	def mapFromGalvo(self, pt):
		'''maps an analog output value to a scene coordinate'''
		p = pt - self.galvo.boundRect.topLeft()
		p.setX(p.x() / scene.galvo.boundRect.width() * self.views()[0].width())
		p.setY(p.y() / scene.galvo.boundRect.height() * self.views()[0].height())
		return p

	def mapToGalvo(self, pt):
		'''maps a scene point to a percent to be used in the galvo driver'''
		p = QtCore.QPointF(self.views()[0].mapFromScene(pt))
		p.setX(p.x() / self.views()[0].width())
		p.setY(p.y() / self.views()[0].height())
		return p

class GalvoBase(QtCore.QThread):
	'''implementation of Galvo Driver that sends one coordinate at a time similar to a QGraphicsObject,
	handles maximum and minimum values accordingly'''
	boundRect = QtCore.QRectF(-10, -10, 20, 20)
	def __init__(self):
		super(GalvoBase, self).__init__()
		self.lasers = (Laser('Laser 1', 0), Laser('Laser 2', 1))
		self.shapes = []
		self.duration = -1

	def setShapes(self, shapes):
		self.shapes = shapes
		if len(shapes) > 0 and not self.isRunning():
			self.start()

	def setBounds(self, boundRect):
		self.boundRect = boundRect
		
class DebugDriver():
	def __getattr__(self, attr):
		print(attr)
		return print

class GalvoDriver(GalvoBase):
	'''implementation of Galvo Driver that sends one coordinate at a time similar to a QGraphicsObject,
	handles maximum and minimum values accordingly'''
	boundRect = QtCore.QRectF(-10, -10, 20, 20)
	def __init__(self):
		super(GalvoDriver, self).__init__()
		self.sample_rate=5000 # Maximum for the NI PCI-6001 is 5kHz.
		self.bufferSize=2
		if not g.DEBUG:
			self.read = int32()
			self.establishChannels()
		else:
			self.read = 'int32'
			self.analog_output = DebugDriver()
			self.digital_output = DebugDriver()

	def establishChannels(self):
		self.analog_output = Task()
		self.analog_output.CreateAOVoltageChan(b'Dev1/ao0',b"",-10.0,10.0,DAQmx_Val_Volts,None)
		self.analog_output.CreateAOVoltageChan(b"Dev1/ao1",b"",-10.0,10.0,DAQmx_Val_Volts,None)
		self.digital_output = Task()
		self.digital_output.CreateDOChan(b'Dev1/port0/line0:7',b"",DAQmx_Val_ChanForAllLines)
		#self.analog_output.CfgSampClkTiming("", 50, DAQmx_Val_Rising, DAQmx_Val_ContSamps, self.bufferSize) 

	def setLaserActive(self, num, active):
		if type(num) in (list, tuple):
			for n in num:
				self.lasers[n].setActive(active)
		else:
			self.lasers[num].setActive(active)
		self.updateDigital()

	def updateDigital(self, active=True):
		digital_data = np.uint8([0, 0, 0, 0, 0, 0, 0, 0])
		if active:
			for i in self.lasers:
				if i.active:
					digital_data[i.pin] = 1
		self.digital_output.WriteDigitalLines(1,1,-1,DAQmx_Val_ChanForAllLines,digital_data,None,None)
		if any(digital_data) and len(self.shapes) > 0 and not self.isRunning():
			self.start()

	def timedDraw(self, path, totalTime):
		pointCount = path.length() / 5
		points = [path.pointAtPercent(i) for i in np.linspace(0., 1., pointCount)]
		pts = []
		for p in points:
			pts.append(self.mapFromPercent(p))
		data = np.array([p.y() for p in pts] + [-p.x() for p in pts], dtype=np.float64) # sent as (y, -x)
		samps = len(data)//2
		self.analog_output.WriteAnalogF64(samps,1,-1,DAQmx_Val_GroupByChannel,data,byref(self.read),None)

	def run(self):
		while len(self.shapes) > 0 and (self.lasers[0].active or self.lasers[1].active):
			if len(self.shapes) > 1:	# draw multiple shapes, jumping between
				for shape in self.shapes:
					self.moveTo(shape[0], penUp=True)
					self.write_points(shape)
			elif len(self.shapes) == 1:	# draw one shape, top to bottom to top again
				self.write_points(self.shapes[0][:-1] + self.shapes[0][::-1])
				
	def mapFromPercent(self, p): 
		p = [self.boundRect.x() + (p.x() * self.boundRect.width()), self.boundRect.y() + (p.y() * self.boundRect.height())]
		return QtCore.QPointF(max(-10, min(p[0], 10)),  max(-10, min(p[1], 10)))

	def moveTo(self, pos, penUp=False):
		pos = self.mapFromPercent(pos)
		self.pos = pos
		if penUp:
			self.updateDigital(active=False)
		data = np.array([pos.y(), pos.y(), -pos.x(), -pos.x()], dtype=np.float64)
		self.analog_output.WriteAnalogF64(self.bufferSize,1,-1,DAQmx_Val_GroupByChannel,data,byref(self.read),None)
		if penUp:
			self.updateDigital()
		#g.ui.statusBar().showMessage("Crosshair at position (%.3f, %.3f)" % (self.pos.x(), self.pos.y()))

	def write_points(self, points):
		self.analog_output.StopTask()
		pts = []
		for p in points:
			pts.append(self.mapFromPercent(p))
		data = np.array([p.y() for p in pts] + [-p.x() for p in pts], dtype=np.float64) # sent as (y, -x)
		samps = len(data)//2
		self.analog_output.WriteAnalogF64(samps,1,-1,DAQmx_Val_GroupByChannel,data,byref(self.read),None)



from qtpy import QtWidgets, QtGui, QtCore
import numpy as np
from . import global_vars as g

class CrossHair(QtWidgets.QGraphicsObject):
	'''draggable crosshair object that acts as an aimer in a QGraphics Scene'''
	sigMoved = QtCore.Signal(object)
	def __init__(self, parent=None, size=7, color = QtCore.Qt.red, pos=QtCore.QPointF(0, 0)):
		QtWidgets.QGraphicsObject.__init__(self, parent)
		self.pen = QtGui.QPen(color)
		self.pen.setWidth(2)
		self.setZValue(10)
		self.setPos(pos)
		self.size = size
		self.dragging = False
		self.newPos = [None, None]
		self.xChanged.connect(lambda : self.setCoord(x=self.pos().x))
		self.yChanged.connect(lambda: self.setCoord(y=self.pos().y))

	def setCoord(self, x=None, y=None):
		if x != None:
			self.newPos[0] = x
		elif y != None:
			self.newPos[1] = y

		if all(self.newPos):
			self.sigMoved.emit(self.pos())
			self.newPos = [None, None]

	def paint(self, painter, option, widget):
		'''paint the crosshair'''
		painter.setPen(self.pen)
		painter.drawEllipse(self.boundingRect())
		painter.drawLine(-self.size, 0, self.size, 0)
		painter.drawLine(0, -self.size, 0, self.size)

	def boundingRect(self):
		'''shape to draw in'''
		return QtCore.QRectF(-self.size - 1, -self.size - 1, 2 * self.size + 2, 2 * self.size + 2)



class GalvoLine(QtWidgets.QGraphicsPathItem):
	def __init__(self, pos):
		self.path = QtGui.QPainterPath(pos)
		super(GalvoLine, self).__init__(self.path)
		self.start = pos
		self.end = pos
		self.path_pen = QtGui.QPen(QtGui.QColor(0, 0, 255))
		self.path_pen.setWidth(2)
		self.setPen(self.path_pen)
		self.setPath(self.path)
		self.mouseIsOver = False

	def paint(self, painter, option, *arg):
		if self.visible:
			painter.setRenderHint(QtGui.QPainter.Antialiasing)
			QtWidgets.QGraphicsPathItem.paint(self, painter, option, *arg)
			painter.setPen(QtGui.QColor(0, 255, 0))
			painter.drawText(self.start.x(), self.start.y(), 'S')
			painter.setPen(QtGui.QColor(255, 0, 0))
			painter.drawText(self.end.x(), self.end.y(), 'E')

	def boundingRect(self):
		newRect = QtWidgets.QGraphicsPathItem.boundingRect(self)
		newRect.adjust(-5, -5, 5, 5)
		return newRect

	def mouseOver(self, pos):
		self.mouseIsOver = self.boundingRect().contains(pos)

	def addPoint(self, p):
		self.path.lineTo(p)
		self.setPath(self.path)
		self.end = p
		self.update()

	def setSelected(self, s):
		self.selected = s
		if self.selected:
			self.path_pen.setColor(QtGui.QColor(255, 0, 0))
		else:
			self.path_pen.setColor(QtGui.QColor(0, 0, 255))
		self.setPen(self.path_pen)
		self.update()

	def rasterPoints(self, count):
		return [self.path.pointAtPercent(i / float(count)) for i in range(count)]

class GalvoStraightLine(QtWidgets.QGraphicsPathItem):
	def __init__(self, posA, posB):
		self.path = QtGui.QPainterPath(posA)
		self.path.lineTo(posB)
		super(GalvoStraightLine, self).__init__(self.path)
		self.start = posA
		self.end = posB
		path_pen = QtGui.QPen(QtGui.QColor(0, 0, 255))
		path_pen.setWidth(2)
		self.setPen(path_pen)
		self.setPath(self.path)
		self.midDraw = False

	def paint(self, painter, option, *arg):
		painter.setRenderHint(QtGui.QPainter.Antialiasing)
		#QtGui.QGraphicsPathItem.paint(self, painter, option, *arg)
		pen = QtGui.QPen(QtGui.QColor(0, 0, 255))
		pen.setWidth(4)
		painter.setPen(pen)
		f = QtGui.QFont('Arial', 13, weight=1)
		f.setBold(True)
		painter.setFont(f)
		painter.drawPoints(*self.rasterPoints())
		painter.setPen(QtGui.QColor(0, 255, 0))
		painter.drawText(self.start.x(), self.start.y(), 'S')
		painter.setPen(QtGui.QColor(255, 0, 0))
		painter.drawText(self.end.x(), self.end.y(), 'E')

	def boundingRect(self):
		newRect = QtWidgets.QGraphicsPathItem.boundingRect(self)
		newRect.adjust(-10, -10, 10, 10)
		return newRect

	def setStart(self, p):
		self.start = p
		self.end = p
		self.path = QtGui.QPainterPath(self.start)
		self.path.lineTo(self.end)
		self.setPath(self.path)
		self.update()
		self.midDraw = True

	def setEnd(self, e, finish=False):
		self.end = e
		self.path = QtGui.QPainterPath(self.start)
		self.path.lineTo(self.end)
		self.setPath(self.path)
		self.update()
		if finish:
			self.midDraw = False

	def rasterPoints(self):
		return [self.path.pointAtPercent(i / (g.intervals - 1.)) for i in range(g.intervals)]

class GalvoShape(GalvoLine):
	def __init__(self, pos):
		super(GalvoShape, self).__init__(pos)

	def paint(self, painter, option, *arg):
		painter.setRenderHint(QtGui.QPainter.Antialiasing)
		QtWidgets.QGraphicsPathItem.paint(self, painter, option, *arg)
		ps = self.rasterPoints()
		if len(ps) > 0:
			painter.drawPolyline(*ps)
			#painter.drawLines(*ps)
			#painter.drawPoints(*ps)

	def mouseOver(self, pos):
		self.mouseIsOver = self.path.contains(pos)
		return self.mouseIsOver

	def close(self):
		self.path.closeSubpath()
		self.setPath(self.path)
		self.update()

	def rasterPoints(self):
		raster = []
		r = self.path.boundingRect()
		r.setHeight(1)
		rect = QtGui.QPainterPath()
		rect.addRect(r)	#create painterPath of the top of the boundRect
		i = 0
		step = g.ui.lineSepCounter.maximum() - g.intervals + 1
		step = g.intervals
		while rect.boundingRect().y() < self.path.boundingRect().bottom():	# while the y-val is above the boundRect
			r = self.path.intersected(rect).boundingRect()
			if i % 2 == 0:
				raster.extend([QtCore.QPointF(x, r.top()) for x in np.arange(r.left(), r.right(), step)])
			else:
				raster.extend([QtCore.QPointF(x, r.top()) for x in np.arange(r.right(), r.left(), -step)])

			rect.translate(0, step) #translate the raster down slightly
			i += 1
		return raster

	def rasterPath(self):
		ps = self.rasterPoints()
		if len(ps) == 0:
			return QtGui.QPainterPath()
		raster = QtGui.QPainterPath(ps[0])
		for p in ps[1:]:
			raster.lineTo(p)
		return raster



from qtpy import QtWidgets, uic, QtCore
import os

class LaserConfigureWindow(QtWidgets.QWidget):
	sigAccepted = QtCore.Signal(list)
	def __init__(self):
		QtWidgets.QWidget.__init__(self)
		uic.loadUi(os.path.join(os.path.dirname(__file__), 'laserConfig.ui'), self)
		self.buttonBox.accepted.connect(self.accepted)
		self.buttonBox.rejected.connect(self.rejected)

	def update(self, lasers):
		self.laser1nameEdit.setText(lasers[0].name)
		self.laser2nameEdit.setText(lasers[1].name)
		self.__dict__['laser1pin%d' % lasers[0].pin].setChecked(True)
		self.__dict__['laser2pin%d' % lasers[1].pin].setChecked(True)

	def accepted(self):
		pin1 = [self.__dict__['laser1pin%d' % i].isChecked() for i in range(8)].index(True)
		pin2 = [self.__dict__['laser2pin%d' % i].isChecked() for i in range(8)].index(True)
		results = [{'name': self.laser1nameEdit.text(), 'pin': pin1}, {'name': self.laser2nameEdit.text(), 'pin': pin2}]
		self.sigAccepted.emit(results)
		self.close()

	def rejected(self):
		self.close()

if __name__ == '__main__':
	app = QtWidgets.QApplication([])
	c = LaserConfigureWindow()
	c.show()
	app.exec_()



from qtpy import QtWidgets, QtCore, QtGui, uic
import sys, time, os
from threading import Timer

from . import global_vars as g
from .config import LaserConfigureWindow
from .galvoDriver import GalvoScene

class GalvoController(QtWidgets.QMainWindow):
	def __init__(self):
		QtWidgets.QMainWindow.__init__(self)
		uic.loadUi(os.path.join(os.path.dirname(__file__), 'galvoController.ui'), self)
		self.scene = GalvoScene()
		g.ui = self
		self.configWindow = LaserConfigureWindow()
		self.configWindow.sigAccepted.connect(self.settingsChanged)
		self.graphicsView.setScene(self.scene)
		self.scene.setSceneRect(QtCore.QRectF(self.graphicsView.rect()))
		self._connect_signals()

	def showEvent(self, ev):
		try:
			g.load_settings()
		except Exception as e:
			print(e)
		self.scene.galvo.setLaserActive((0, 1), False)

	def closeEvent(self, ev):
		self.scene.galvo.setLaserActive((0, 1), False)
		if self.actionAutosave.isChecked():
			g.save_settings()
		sys.exit(0)

	def showHelp(self):
		self.helpText = QtWidgets.QTextBrowser()
		self.helpText.setOpenExternalLinks(True)
		self.helpText.setHtml("""
<h1 style="width:100%; height:20%; text-align:center;">Galvonmeter Controller</h1>
<div style="width:90%; height: 70%; margin:5%; background: #CCCCCC">
	<p style="padding:10px"> The Galvonmeter control software communicates with a 2 dimensional laser control system via the National Instruments USB 6001</p>
	<p>The pin information for the device can be found in the Laser Configuration Dialog.

	The program will automatically run in DEBUG mode if the Data Acquisition device is not detected. Once the device is detected, there are a few simple steps:
	<ul>
	  <li>Ensure the pins are set correctly (and optionally name your lasers), by going to Settings>Configure Ports</li>
	  <li>Toggle the lasers with the large named buttons to ensure both work correctly</li>
	  <li>Relocate and resize the window to span over your view</li>
	  <li>Set the opacity of the window so you can see the image behind it</li>
	  <li>Calibrate the controller so that the laser lies beneath the red crosshair</li>
	</ul>
	</p>
	<br>
	<p>Once the calibration is complete, use the mouse to move the crosshair, or draw a line or ROI to specify a path for the laser<p>
</div>
			""")
		self.helpText.setReadOnly(True)
		self.helpText.show()
		self.helpText.window().resize(700, 500)

	def _connect_signals(self):

		self.actionShow_Help.triggered.connect(self.showHelp)
		self.clearButton.pressed.connect(self.scene.clear)
		self.closeButton.pressed.connect(self.close)

		self.laser1Button.pressed.connect(lambda : self.pressed(0))
		self.laser1Button.released.connect(lambda : self.released(0))
		self.pulseButton.pressed.connect(lambda : self.pulse(0))

		self.laser2Button.pressed.connect(lambda : self.pressed(1))
		self.laser2Button.released.connect(lambda : self.released(1))
		self.pulse2Button.pressed.connect(lambda : self.pulse(1))

		self.roiButton.pressed.connect(lambda : self.scene.setDrawMethod('ROI'))
		self.lineButton.pressed.connect(lambda : self.scene.setDrawMethod('Line'))
		self.pointButton.pressed.connect(lambda : self.scene.setDrawMethod('Point'))
		self.lineSepCounter.valueChanged.connect(lambda v: setattr(g, 'intervals', v))
		self.lineSepCounter.valueChanged.connect(lambda v: [roi.update() for roi in self.scene.rois] if self.scene.drawMethod == 'ROI' else self.scene.line.update())

		self.opacitySlider.valueChanged.connect(lambda v: self.setWindowOpacity(v/100.))
		self.opacitySlider.setValue(85)

		self.actionConfigure.triggered.connect(self.configure)
		self.actionCalibrate.triggered.connect(self.calibrate)
		self.actionReset.triggered.connect(self.reset)
		self.actionDisconnect.triggered.connect(lambda : sys.exit(0))
		self.actionImport.triggered.connect(lambda : g.import_settings(QtWidgets.QFileDialog.getOpenFileName(filter='Pickled files (*.p)')))
		self.actionExport.triggered.connect(lambda : g.export_settings(QtWidgets.QFileDialog.getSaveFileName(filter='Pickled files (*.p)')))
		
		self.scene.galvo.lasers[0].sigRenamed.connect(self.laser1Button.setText)
		self.scene.galvo.lasers[1].sigRenamed.connect(self.laser2Button.setText)

	def configure(self):
		self.configWindow.update(self.scene.galvo.lasers)
		self.configWindow.show()

	def settingsChanged(self, lasers):
		self.scene.galvo.lasers[0].rename(lasers[0]['name'])
		self.scene.galvo.lasers[1].rename(lasers[1]['name'])
		self.scene.galvo.lasers[0].changePin(lasers[0]['pin'])
		self.scene.galvo.lasers[1].changePin(lasers[1]['pin'])

	def reset(self):
		self.pointButton.click()
		self.scene.center()
		self.scene.resetBounds()

	def calibrate(self):
		self.setFixedSize(self.size())
		self.reset()
		aimRect = QtCore.QRectF(self.graphicsView.mapToScene(0, 0), self.graphicsView.mapToScene(20, 20))
		aim = QtWidgets.QGraphicsRectItem(aimRect)
		aim.setBrush(QtGui.QColor(255, 0, 0))
		self.scene.addItem(aim)
		font = QtGui.QFont("Helvetica", 15, 3)
		ti = self.scene.addText('Position the laser in optimal top\n   left, then press any key', font = font)
		self.scene.keyPressEvent = lambda ev: setattr(self.scene, 'tempRect', QtCore.QRectF(self.scene.galvo.pos, QtCore.QSizeF()))
		while not hasattr(self.scene, 'tempRect'):
			app.processEvents()
			time.sleep(.01)
		aimRect.moveTo(self.graphicsView.mapToScene(self.graphicsView.width() - 20, self.graphicsView.height() - 20))
		aim.setRect(aimRect)
		self.scene.keyPressEvent = lambda ev: self.scene.tempRect.setSize(QtCore.QSizeF(self.scene.galvo.pos.x() - self.scene.tempRect.x(), self.scene.galvo.pos.y() - self.scene.tempRect.y()))
		ti.setPlainText('Now position the laser in optimal bottom \n      right, and press a key')
		while self.scene.tempRect.isEmpty():
			app.processEvents()
			time.sleep(.01)
		self.scene.removeItem(aim)
		self.scene.keyPressEvent = lambda ev: GalvoScene.keyPressEvent(self.scene, ev)
		self.scene.galvo.setBounds(self.scene.tempRect)
		del self.scene.tempRect
		self.scene.removeItem(ti)
		self.setMaximumSize(2000,2000);
		self.setMinimumSize(0,0);

	def pulse(self, num):
		laser = self.scene.galvo.lasers[num]
		if any([laser.active for laser in self.scene.galvo.lasers]):
			return
		pulse_time = self.pulseSpin.value() / 1000.0
		if self.scene.drawMethod == 'Line':
			pts = [self.scene.mapToGalvo(p) for p in self.scene.line.rasterPoints()]
			timers = [Timer(i * pulse_time/g.intervals, self.scene.galvo.moveTo, args = (pts[i],)) for i in range(len(pts))]
			timers.append(Timer(pulse_time, self.scene.galvo.setLaserActive, args = (num, False)))
			self.scene.galvo.setLaserActive(num, True)
			[ti.start() for ti in timers]
		elif self.scene.drawMethod == 'ROI':
			ps = []
			for roi in self.scene.rois:
				ps.extend([self.scene.mapToGalvo(p) for p in roi.rasterPoints()])
			self.scene.galvo.setLaserActive(num, True)
			Timer(pulse_time, self.scene.galvo.setLaserActive, args=(num, False)).start()
			self.scene.galvo.write_points(ps)
		else:
			self.scene.galvo.setLaserActive(num, True)
			Timer(pulse_time, self.scene.galvo.setLaserActive, args=(num, False)).start()

	def pressed(self, num):
		laser = self.scene.galvo.lasers[num]
		button = [self.laser1Button, self.laser2Button][num]
		button.presstime = time.time()
		if not button.isChecked():
			if self.scene.drawMethod == 'Point':
				self.scene.galvo.setLaserActive(num, True)

	def released(self, num):
		laser = self.scene.galvo.lasers[num]
		button = [self.laser1Button, self.laser2Button][num]
		if time.time() - button.presstime > .2: # used as manual button
			button.setChecked(False)
			self.scene.galvo.setLaserActive(num, False)
		elif not button.isChecked():
			self.scene.galvo.setLaserActive(num, button.isChecked())

def main():
	app = QtWidgets.QApplication([])
	controller = GalvoController()
	controller.show()
	app.exec_()

if __name__ == '__main__':
	app = QtWidgets.QApplication([])
	controller = GalvoController()
	controller.show()
	app.exec_()
